package pen_flowchart;

import java.awt.Dimension;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
//import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
//import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JOptionPane;

import org.sourceforge.jlibeps.epsgraphics.EpsGraphics2D;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

// Need org.sourceforge.jlibeps.epsgraphics
// http://jlibeps.sourceforge.net/

// Need com.itextpdf


public class Flowchart {
	private Parts start, end;
	private List<Parts> parts;
	private Parts clipboard_parts;
	private boolean dirty;
	private FlowchartPanel panel;
	private Parts_LoopBegin hilightLoopBegin;
	private Parts_LoopEnd   hilightLoopEnd;
	int x0,y0;

	public Flowchart(FlowchartPanel panel)
	{
		this.panel = panel;
		reset();
	}

	public void reset()
	{
		clipboard_parts = null;
		Parts.resetSize();
		parts = new ArrayList<Parts>();
		start = new Parts_Terminal();
		start.setText("はじめ");
		end = new Parts_Terminal();
		end.setText("おわり");
		Parts_Bar bar = new Parts_Bar();
		start.setLeft(bar);
		bar.setLeft(end);
		x0 = 200;
		y0 = 20;
		dirty = false;
		hilightLoopBegin = null;
		hilightLoopEnd = null;
		findParts();
//		repaint();
	}

	public boolean isDirty()
	{
		return dirty;
	}

	public void makeClean()
	{
		if(dirty)
		{
			dirty = false;
			panel.setTitle();
		}
		else
			dirty = false;
	}

	public void makeDirty()
	{
		if(!dirty)
		{
			dirty = true;
			panel.setTitle();
		}
		else
			dirty = true;
	}

	public void setOrigin(int x, int y)
	{
		x0 = x;
		y0 = y;
	}

	public void moveOrigin(int dx, int dy)
	{
		x0 += dx;
		y0 += dy;
	}

	public void repaint()
	{
		findParts();
		panel.repaint();
		panel.refreshCode();
	}

	public void repaint_norefresh()
	{
		findParts();
		panel.repaint();
//		panel.refreshCode();
	}


	public void moveToClipboard(Parts p)
	{
		clipboard_parts = p;
	}

	public Parts getFromClipboard()
	{
		return clipboard_parts;
	}

	private void findParts()
	{
		parts.clear();
		start.makeBarList(parts);
	}

	public Parts findParent(Parts p)
	{
		return findParent(start, p);
	}

	private Parts findParent(Parts p0, Parts p)
	{
		Parts rtnv = null;
		if(p0.getLeft() != null)
		{
			if(p0.getLeft() == p) return p0;
			rtnv = findParent(p0.getLeft(), p);
			if(rtnv != null) return rtnv;
		}
		if(p0.getRight() != null)
		{
			rtnv = findParent(p0.getRight(), p);
		}
		return rtnv;
	}

	public boolean addParts(String s, Point p)
	{
		Parts parts_nearest = null;
		int least_distance = Parts_Bar.near_distance;
		for(Parts part : parts)
		{
			int distance = part.getX1() - p.x;
			if(distance < 0) distance = - distance;
			if(part instanceof Parts_Bar
					&& part.inside(p.x, p.y)
					&& distance <  least_distance)
			{
				parts_nearest = part;
				least_distance = distance;
			}
		}
		if(parts_nearest != null) return addParts(s, parts_nearest);
		return false;
	}

	public boolean addParts(String s, Parts b)
	{
		if(s.equals("SUBSTITUTE"))
		{
			Parts_Substitute p = new Parts_Substitute();
			Parts_Bar b1 = new Parts_Bar();
			p.setLeft(b1);
			b.connectLeft(p);
			p.connectLeft(b1);
		}
		else if(s.equals("MISC") || s.equals("GRAPHICS"))
		{
			Parts_Misc p = new Parts_Misc();
			Parts_Bar b1 = new Parts_Bar();
			p.setLeft(b1);
			b.connectLeft(p);
			p.connectLeft(b1);
		}
		else if(s.equals("INPUT"))
		{
			Parts_Input p = new Parts_Input();
			Parts_Bar b1 = new Parts_Bar();
			p.setLeft(b1);
			b.connectLeft(p);
			p.connectLeft(b1);
		}
		else if(s.equals("OUTPUT"))
		{
			Parts_Output p = new Parts_Output();
			Parts_Bar b1 = new Parts_Bar();
			p.setLeft(b1);
			b.connectLeft(p);
			p.connectLeft(b1);
		}
		else if(s.equals("IF"))
		{
			Parts_If p = new Parts_If();
			Parts_Bar b1 = new Parts_Bar();
			Parts_Bar b2 = new Parts_Bar();
			Parts_Bar b3 = new Parts_Bar();
			Parts_Null n = new Parts_Null();
			p.setEndif(n);
			b.connectLeft(n);
			n.connectLeft(b3);
			p.setLeft(b1);
			p.setRight(b2);
			b1.setLeft(n);
			b2.setLeft(n);
			b.setLeft(p);
		}
		else if(s.equals("LOOP1"))
		{
			Parts_LoopBegin lb = new Parts_LoopBegin1();
			Parts_LoopEnd   le = new Parts_LoopEndEmpty();
			Parts_Bar b1 = new Parts_Bar();
			Parts_Bar b2 = new Parts_Bar();
			lb.setLoopEnd(le);
			le.setLoopBegin(lb);
			lb.setLeft(b1);
			b1.setLeft(le);
			le.setLeft(b2);
			b2.setLeft(b.getLeft());
			b.setLeft(lb);
		}
		else if(s.equals("LOOP2"))
		{
			Parts_LoopBegin lb = new Parts_LoopBegin2();
			Parts_LoopEnd   le = new Parts_LoopEnd2();
			Parts_Bar b1 = new Parts_Bar();
			Parts_Bar b2 = new Parts_Bar();
			lb.setLoopEnd(le);
			le.setLoopBegin(lb);
			lb.setLeft(b1);
			b1.setLeft(le);
			le.setLeft(b2);
			b2.setLeft(b.getLeft());
			b.setLeft(lb);
		}
		else if(s.equals("LOOPINC"))
		{
			Parts_LoopBegin lb = new Parts_LoopBeginInc();
			Parts_LoopEnd   le = new Parts_LoopEndEmpty();
			Parts_Bar b1 = new Parts_Bar();
			Parts_Bar b2 = new Parts_Bar();
			lb.setLoopEnd(le);
			le.setLoopBegin(lb);
			lb.setLeft(b1);
			b1.setLeft(le);
			le.setLeft(b2);
			b2.setLeft(b.getLeft());
			b.setLeft(lb);

		}
		else if(s.equals("LOOPDEC"))
		{
			Parts_LoopBegin lb = new Parts_LoopBeginDec();
			Parts_LoopEnd   le = new Parts_LoopEndEmpty();
			Parts_Bar b1 = new Parts_Bar();
			Parts_Bar b2 = new Parts_Bar();
			lb.setLoopEnd(le);
			le.setLoopBegin(lb);
			lb.setLeft(b1);
			b1.setLeft(le);
			le.setLeft(b2);
			b2.setLeft(b.getLeft());
			b.setLeft(lb);
		}
		else return false;

		makeDirty();
		repaint();
		return true;
	}

	public void paint(Graphics g)
	{
		Point p = new Point(), p1 = new Point(), p2 = new Point();
		g.setFont(new Font(Font.MONOSPACED, Font.PLAIN, Parts.size * 5 / 2));
		FontMetrics fm = g.getFontMetrics();
		start.calcSize(g, fm, p, p1, p2);
		Dimension panelsize = panel.getSize();
		if(x0 + p1.x > panelsize.width - 5) x0 = panelsize.width - 5 - p1.x;
		if(x0 + p2.x < 5) x0 = 5 - p2.x;
		if(y0 + p1.y > panelsize.height - 5) y0 = panelsize.height - 5 - p1.y;
		if(y0 + p2.y < 5) y0 = 5 - p2.y;
		Point p0 = new Point(x0 , y0);
		start.paint(g, fm, p0);
	}

	public void paint(Graphics g, VariablePanel variablePanel)
	{
		Point p = new Point(), p1 = new Point(), p2 = new Point();
		g.setFont(new Font(Font.MONOSPACED, Font.PLAIN, Parts.size * 5 / 2));
		FontMetrics fm = g.getFontMetrics();
		start.calcSize(g, fm, p, p1, p2);
		Dimension panelsize = panel.getSize();
		if(x0 + p1.x > panelsize.width - 5) x0 = panelsize.width - 5 - p1.x;
		if(x0 + p2.x < 5) x0 = 5 - p2.x;
		if(y0 + p1.y > panelsize.height - 5) y0 = panelsize.height - 5 - p1.y;
		if(y0 + p2.y < 5) y0 = 5 - p2.y;
		Point p0 = new Point(x0 , y0);
		start.paint_variables(g, fm, p0, variablePanel);
		start.paint(g, fm, p0);
	}

	public void rightButtonClicked(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		for(Parts part : parts)
		{
			if(part.inside(x,y))
			{
				part.dispMenu(this, e);
				return;
			}
		}
	}

	public void doubleClicked(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		for(Parts part : parts)
		{
			if(part.getY1() <= y && part.getY2() >= y
					&& part.getX1() <= x && part.getX2() >= x)
			{
				if(part.editMe())
				{
					makeDirty();
					repaint();
				}
				return;
			}
		}

	}

	public void mouseMoved(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		for(Parts part : parts)
		{
			if(part.getY1() <= y && part.getY2() >= y
					&& part.getX1() <= x && part.getX2() >= x)
			{
				if(part instanceof Parts_LoopBegin && hilightLoopBegin == null)
				{
					hilightLoopBegin = (Parts_LoopBegin)part;
					hilightLoopEnd = hilightLoopBegin.getLoopEnd();
					setHilight(true);
				}
				else if(part instanceof Parts_LoopEnd && hilightLoopEnd == null)
				{
					hilightLoopEnd = (Parts_LoopEnd)part;
					hilightLoopBegin = hilightLoopEnd.getLoopBegin();
					setHilight(true);
				}
				return;
			}
		}
		if(hilightLoopBegin != null || hilightLoopEnd != null)
			setHilight(false);
	}

	private void setHilight(boolean b)
	{
		if(hilightLoopBegin != null) hilightLoopBegin.setHilight(b);
		if(hilightLoopEnd != null) hilightLoopEnd.setHilight(b);
		if(!b)
		{
			hilightLoopBegin = null;
			hilightLoopEnd   = null;
		}
		repaint_norefresh();

	}

	public Parts getFirstParts()
	{
		return start.getLeft();
	}

	public void appendParts(Node node)
	{
		appendParts(start.getLeft(), node);
		findParts();
	}

	private void appendParts(Parts parts, Node node)
	{
		for(Node child = node.getFirstChild(); child != null; child = child.getNextSibling())
		{
			String name = child.getNodeName();
			NamedNodeMap map = child.getAttributes();
			if(name == "substitute")
			{
				Node s1 = map.getNamedItem("var");
				Node s2 = map.getNamedItem("val");
				Parts_Substitute p1 = new Parts_Substitute();
				p1.setValue(s1.getNodeValue(), s2.getNodeValue());
				parts.connectLeft(p1);
				parts = new Parts_Bar();
				p1.connectLeft(parts);
			}
			else if(name == "misc" || name == "graphics")
			{
				Parts_Misc p1 = new Parts_Misc();
				Node command = map.getNamedItem("command");
				Node argc_node = map.getNamedItem("argc");
				int argc = 0;
				String[] argv = null;
				if(argc_node != null)
				{
					argc = Integer.parseInt(argc_node.getNodeValue());
					argv = new String[argc];
					for(int i = 0; i < argc; i++)
					{
						Node n = map.getNamedItem("argv" + Integer.toString(i + 1));
						argv[i] = n == null ? new String() : new String(n.getNodeValue());
					}
				}
				p1.setValue(command.getNodeValue(), argv);
				parts.connectLeft(p1);
				parts = new Parts_Bar();
				p1.connectLeft(parts);
			}
			else if(name == "input")
			{
				Node s1 = map.getNamedItem("var");
				Parts_Input p1 = new Parts_Input();
				p1.setValue(s1.getNodeValue());
				parts.connectLeft(p1);
				parts = new Parts_Bar();
				p1.connectLeft(parts);
			}
			else if(name == "output")
			{
				Node s1 = map.getNamedItem("val");
				Node s2 = map.getNamedItem("nocr");
				Parts_Output p1 = new Parts_Output();
				p1.setValue(s1.getNodeValue(), s2 != null ? s2.getNodeValue().equals("true") : false);
				parts.connectLeft(p1);
				parts = new Parts_Bar();
				p1.connectLeft(parts);
			}
			else if(name == "if")
			{
				Node s1 = map.getNamedItem("condition");
				Parts_If p1 = new Parts_If();
				Parts_Null p2 = new Parts_Null();
				Parts_Bar b1 = new Parts_Bar();
				Parts_Bar b2 = new Parts_Bar();
				Parts_Bar b3 = new Parts_Bar();
				p1.setEndif(p2);
				p1.setValue(s1.getNodeValue());
				parts.connectLeft(p1);
				p1.connectLeft(b1);
				b1.connectLeft(p2);
				p2.connectLeft(b3);
				p1.setRight(b2);
				b2.setLeft(p2);
				for(Node grandchild = child.getFirstChild(); grandchild != null; grandchild = grandchild.getNextSibling())
				{
					if(grandchild.getNodeName() == "then")
						appendParts(b1, grandchild);
					else if(grandchild.getNodeName() == "else")
						appendParts(b2, grandchild);
				}
				parts = b3;
			}
			else if(name == "while")
			{
				Node s1 = map.getNamedItem("condition");
				Parts_LoopBegin1 p1 = new Parts_LoopBegin1();
				Parts_LoopEndEmpty p2 = new Parts_LoopEndEmpty();
				p1.setLoopEnd(p2);
				p2.setLoopBegin(p1);
				Parts_Bar b1 = new Parts_Bar();
				Parts_Bar b2 = new Parts_Bar();
				p1.setValue(s1.getNodeValue());
				parts.connectLeft(p1);
				p1.connectLeft(b1);
				b1.connectLeft(p2);
				p2.connectLeft(b2);
				appendParts(b1, child);
				parts = b2;
			}
			else if(name == "until")
			{
				Node s1 = map.getNamedItem("condition");
				Parts_LoopBegin2 p1 = new Parts_LoopBegin2();
				Parts_LoopEnd2 p2 = new Parts_LoopEnd2();
				p1.setLoopEnd(p2);
				p2.setLoopBegin(p1);
				Parts_Bar b1 = new Parts_Bar();
				Parts_Bar b2 = new Parts_Bar();
				p2.setValue(s1.getNodeValue());
				parts.connectLeft(p1);
				p1.connectLeft(b1);
				b1.connectLeft(p2);
				p2.connectLeft(b2);
				appendParts(b1, child);
				parts = b2;
			}
			else if(name == "loopinc")
			{
				Node s1 = map.getNamedItem("var");
				Node s2 = map.getNamedItem("start");
				Node s3 = map.getNamedItem("end");
				Node s4 = map.getNamedItem("step");
				Parts_LoopBeginInc p1 = new Parts_LoopBeginInc();
				Parts_LoopEndEmpty p2 = new Parts_LoopEndEmpty();
				p1.setLoopEnd(p2);
				p2.setLoopBegin(p1);
				Parts_Bar b1 = new Parts_Bar();
				Parts_Bar b2 = new Parts_Bar();
				p1.setValue(s1.getNodeValue(), s2.getNodeValue(), s3.getNodeValue(), s4.getNodeValue());
				parts.connectLeft(p1);
				p1.connectLeft(b1);
				b1.connectLeft(p2);
				p2.connectLeft(b2);
				appendParts(b1, child);
				parts = b2;

			}
			else if(name == "loopdec")
			{
				Node s1 = map.getNamedItem("var");
				Node s2 = map.getNamedItem("start");
				Node s3 = map.getNamedItem("end");
				Node s4 = map.getNamedItem("step");
				Parts_LoopBeginDec p1 = new Parts_LoopBeginDec();
				Parts_LoopEndEmpty p2 = new Parts_LoopEndEmpty();
				p1.setLoopEnd(p2);
				p2.setLoopBegin(p1);
				Parts_Bar b1 = new Parts_Bar();
				Parts_Bar b2 = new Parts_Bar();
				p1.setValue(s1.getNodeValue(), s2.getNodeValue(), s3.getNodeValue(), s4.getNodeValue());
				parts.connectLeft(p1);
				p1.connectLeft(b1);
				b1.connectLeft(p2);
				p2.connectLeft(b2);
				appendParts(b1, child);
				parts = b2;

			}
		}
	}

	public void appendCode(StringBuilder sb)
	{
		start.appendTo(sb, 0);
	}



	public String getEps(FileOutputStream s, VariablePanel variablePanel) {
		EpsGraphics2D g = new EpsGraphics2D();
		int size = Parts.getSize();
		Parts.resetSize();
		paint(g, variablePanel);
		Parts.setSize(size);
		return g.toString();
	}
/*
	public void writePdf(FileOutputStream s) {
		int xsize, ysize;
		EpsGraphics2D g1 = new EpsGraphics2D();// 寸法計算のためだけに作る。
		Point p = new Point(), p1 = new Point(), p2 = new Point();
		FontMetrics fm1 = g1.getFontMetrics();
		start.calcSize(g1, fm1, p, p1, p2);
		g1.dispose(); // そんなわけなので捨てる
		xsize = p2.x - p1.x; //このxsize, ysizeのためだけにEpsGraphicsを作ったのだった
		ysize = p2.y - p1.y;
		BufferedOutputStream bout = new BufferedOutputStream(s);
		com.itextpdf.text.Document doc = new com.itextpdf.text.Document(new com.itextpdf.text.Rectangle(0,0,xsize + 20, ysize + 10));
		com.itextpdf.text.pdf.PdfWriter writer;
		try {
			writer = com.itextpdf.text.pdf.PdfWriter.getInstance(doc, bout);
			doc.open();
			com.itextpdf.text.pdf.PdfContentByte cb = writer.getDirectContent();
//			cb.concatCTM(1,0,0,1,0,0); //デフォルトが1 0 0 1だろうから不要だろうなあ。
			Graphics2D g = cb.createGraphics(xsize + 20, ysize + 10);
			FontMetrics fm = g.getFontMetrics();
			Point p0 = new Point(-p1.x +10, 5);
			doc.setPageSize(new com.itextpdf.text.Rectangle(0,0,(p2.x - p1.x + 20), (p2.y - p1.y + 10)));
			start.paintPdf(cb, p2.y - p1.y + 10, g, fm, p0);
			g.dispose();
		} catch (com.itextpdf.text.DocumentException e) {
			e.printStackTrace();
		}
		finally
		{
			doc.close();
		}

	}
*/

	public void makeXML(Document doc, Element root) {
		start.makeXML(doc, root);
	}

	public void savePng(String filename_e, VariablePanel variablePanel) {
		int size = Parts.getSize();
		BufferedImage image = new BufferedImage(1, 1, BufferedImage.TYPE_INT_RGB);
		Graphics2D g = image.createGraphics();
		Parts.setSize(12);
		Font font = new Font(Font.SANS_SERIF, Font.PLAIN, Parts.size * 5 / 2);
		g.setFont(font);
		Point p = new Point(), p1 = new Point(), p2 = new Point();
		FontMetrics fm = g.getFontMetrics();
		start.calcSize(g, fm, p, p1, p2);
		int v_width = ((Parts_Terminal)start).get_variables_width(g, fm, variablePanel);
		if(p2.x - p1.x - start.getX2() < v_width) p2.x = p1.x + start.getX2() + v_width;
		image = new BufferedImage(p2.x - p1.x + 2 , p2.y - p1.y + 2, BufferedImage.TYPE_INT_RGB);
		g = image.createGraphics();
		g.setFont(font);
		g.setStroke(new BasicStroke(2.0f));
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, p2.x - p1.x + 2, p2.y - p1.y + 2);
		Point point = new Point(-p1.x, 1);
		start.paint_variables(g, fm, point, variablePanel);
		start.paint(g, fm, point);
		try {
			ImageIO.write(image, "png", new File(filename_e));
		} catch (IOException e) {
			JOptionPane.showConfirmDialog(panel, "File I/O Error", "PEN",
					JOptionPane.OK_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
		} catch (Exception e)
		{
			JOptionPane.showConfirmDialog(panel, "File I/O Error", "PEN",
					JOptionPane.OK_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
		}

		Parts.setSize(size);
	}


}
